# -*- coding: mbcs -*-

#abaqus cae noGUI=SCRIPT_main.py
#   200s

from operator import ge
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import * 
from load import *
from mesh import *
from optimization import *
from job import * 
from sketch import * 
from visualization import *
from connectorBehavior import *
from regionToolset import *
import odbAccess
import sys
import getpass
import time
import os
#endregionex
if getpass.getuser() == "egilm":
    script_folder = 'C:\\Users\\egilm\\NTNU\\o365_Masteroppgave2022 - General\ABAQUS_python_model'
    runoptions_key="egil"
elif getpass.getuser() == "Egil":
    script_folder = 'C:\\Users\\Egil\\NTNU\\o365_Masteroppgave2022 - General\\ABAQUS_python_model'
    runoptions_key="egil"
else:
    script_folder = 'C:\\Users\\emili\\NTNU\\o365_Masteroppgave2022 - ABAQUS_python_model'
    runoptions_key="emilie"
sys.path.append(script_folder)
from CONF_config                import *
import RES_process_results
import UTIL_assembly
import UTIL_import 
import UTIL_model
import UTIL_mesh
import SCRIPT_generate_geometry
import SCRIPT_assign_properties


RUN_OPTIONS={
    "egil":
        {
        "Generate geometry"     : False,
        "Load model"            : False,
        "Update properties"     : True,
        "Update section"        : False,
        "Update mesh"           : False,
        "Update step/loads"     : False,
        "Submit job"            : True,
        },
    "emilie":
        {
        "Generate geometry"     : False,
        "Load model"            : True,
        "Update properties"     : True,
        "Update section"        : False,
        "Update mesh"           : True,
        "Update step/loads"     : True,
        "Submit job"            : True,
        },
}
run_options= RUN_OPTIONS[runoptions_key]

def main():
    UTIL_model.script_header("Running main script")
    UTIL_model.log_progress(script_folder,"Running main script for "+MODELNAME)

    if run_options["Generate geometry"]:
        UTIL_model.section_header("Generating geometry",start_time)
        model=SCRIPT_generate_geometry.generate_geometry(input_file,False)
    elif run_options["Load model"]:
        openMdb(pathName=script_folder+"\\pre_generated_models\\"+MODELNAME)
        model=UTIL_model.get_model()
    else:
        model=UTIL_model.get_model()


    if(run_options["Update properties"]):
        UTIL_model.section_header("Updating properties",start_time)
        SCRIPT_assign_properties.assign_properties(input_file,run_options["Update section"])
    imported_model_data=UTIL_import.import_model_data(model,input_file)
    imported_model_data['All building elements'] = UTIL_assembly.reassociate_assembly_items(model,imported_model_data['All building elements'],MODIFIED_SUFFIX)  
    sensor_data,mode_bib=UTIL_import.import_sensor_data_and_mode_shapes("MAC", input_file, START_ROW)


    if(run_options["Update mesh"]):
        UTIL_model.section_header("Mesh",start_time)
        UTIL_model.section_header("Refreshing part and instance associations",start_time)   
        UTIL_mesh.flush_mesh(model)
        UTIL_mesh.seed_instance_bib(
            current_model           = model,
            instance_bib            = imported_model_data["All building elements"],
            )
        UTIL_mesh.generate_mesh(model)
    UTIL_assembly.set_mode_nodes(
        sensor_data,
        model,
        imported_model_data["Floors"]
        )
    if(run_options["Update step/loads"]):
        UTIL_model.section_header("Step",start_time)
        UTIL_model.create_steps(model,STEPS)
        UTIL_model.section_header("Loads",start_time)
        UTIL_model.create_loads(model)
    UTIL_model.create_job(
        name_of_job   = 'Job',
        name_of_model = MODELNAME
        )
    if(run_options["Submit job"]):
        UTIL_model.section_header("Job",start_time)
        UTIL_model.submit_job(name_of_job='Job')
    UTIL_model.section_header("Processsing results",start_time)
    outputdb = odbAccess.openOdb('Job'+'.odb')
    RES_process_results.save_job_result_to_file(
        script_folder,
        MODELNAME,
        RES_process_results.extract_eigfrecs(outputdb,"Job","Complex frequency"),
        RES_process_results.MAC(outputdb,mode_bib,"Job"),
        model,
        )
    UTIL_model.section_header("Opertation finished",start_time)
    UTIL_model.log_progress(script_folder,"\tOperation finished")
    outputdb.close()

if __name__ == '__main__':
    start_time=time.time()
    input_file= script_folder+'\\Input\\'+MODELNAME+'.xlsx'    
    main()
