import openpyxl
from CONF_config                import *
from UTIL_model                 import *
from CLASS_classes              import *
import UTIL_modelbuild
from UTIL_connectors            import *
from UTIL_material              import *
from UTIL_section               import *
def import_model_data(model,input_file):
    imported_model_data={}
    imported_model_data["Layout"]                = import_layout_bib(
            "Layout", 
            input_file)
    imported_model_data["Walls"]                = import_wall_bib(
            sheet_name      ='Walls', 
            wb_name         =input_file, 
            start_row       =START_ROW,
            current_model   =model,
            building_layout =imported_model_data["Layout"]
            )
    imported_model_data["Floors"]               = import_floor_bib(
            sheet_name      ='Floors',
            wb_name         =input_file,
            start_row       =START_ROW,
            current_model   =model,
            building_layout =imported_model_data["Layout"]
            )
    imported_model_data["ColumnsBeams"]         = import_columnbeam_bib(
            sheet_name      ='ColumnsBeams',
            wb_name         =input_file,
            start_row       =START_ROW,
            current_model   =model,
            )
    imported_model_data["All shell elements"]   = dict((key, value) for d in (imported_model_data["Walls"], imported_model_data["Floors"]) for key, value in d.items())
    imported_model_data["All building elements"]= dict((key, value) for d in (imported_model_data["Walls"], imported_model_data["Floors"],imported_model_data["ColumnsBeams"]) for key, value in d.items())
    imported_model_data["Openings"]             = import_opening_types(
            sheet_name ="Openings", 
            wb_name = input_file, 
            start_row = START_ROW, 
            current_model =model
            )
    imported_model_data["OpeningPlacement"]     = import_opening_placements(
        sheet_name ="OpeningPlacement", 
        wb_name = input_file, 
        start_row = START_ROW, 
        current_model =model,
        opening_type_bib=imported_model_data["Openings"]
            )
    imported_model_data["Connectors"]       = import_connectors("Connectors", input_file, START_ROW, model)
    imported_model_data["Connector density"]= import_conenctor_density("Connector density",input_file,START_ROW)
    imported_model_data["Materials"]        = import_material_bib("Materials", input_file, START_ROW, model)
    imported_model_data["Shell sections"]   = import_shell_sections("Shell sections", input_file, START_ROW, model)
    imported_model_data["Wire sections"]    = import_wire_sections("Wire sections", input_file, START_ROW, model)
    return imported_model_data

def format_excel_row(row_data,excel_submap):
        formated_data={}
        for colname,(data_type,col_num) in excel_submap.items():
                if row_data[col_num] is not None:
                        if data_type=="float":
                                formated_data[colname] = float(row_data[col_num])
                        elif data_type=="int":
                                formated_data[colname] = int(row_data[col_num])
                        elif data_type=="str":
                                formated_data[colname] = str(row_data[col_num])
                        elif data_type=="bool":
                                formated_data[colname] = bool(row_data[col_num])
        return formated_data

def import_wall_bib(sheet_name, wb_name, start_row, current_model,building_layout):
    workbook = openpyxl.load_workbook(wb_name, data_only=True)
    sheet = workbook[sheet_name]
    wall_bib = {}
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=len(EXCEL_MAP["Walls"].keys()), values_only=True):
        if(row[0]):
            wall_bib=UTIL_modelbuild.WallTools.instantiate_wall_array(
                current_model       = current_model,
                building_layout     = building_layout,
                wall_bib            = wall_bib,
                wall_data           = format_excel_row(row,EXCEL_MAP["Walls"]),
                )
    workbook.close()
    return wall_bib


def import_opening_types(sheet_name =[], wb_name = [], start_row = [], current_model =[]):
    workbook = openpyxl.load_workbook(wb_name, data_only = True)
    sheet = workbook[sheet_name]
    opening_type_bib={}
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=4, values_only=True):
        if (row[0]):
            opening_type_bib[str(row[0])]=Opening(
                model               = current_model,
                part                = None,
                partname            = str(row[0]),
                instance            = None,
                instancename        = None,
                origin              = (0,0,0),
                normal              = (0,0,0),
                opening_dimension   = (int(row[1]),int(row[2]))
            )
    workbook.close()
    return opening_type_bib

def import_opening_placements(
        sheet_name =[], 
        wb_name = [], 
        start_row = [], 
        current_model =[],
        opening_type_bib=[]
    ):
    opening_bib={}

    workbook = openpyxl.load_workbook(wb_name, data_only = True)
    sheet = workbook[sheet_name] 
    for idx,row in enumerate(sheet.iter_rows(min_row=start_row, min_col=1, max_col=6, values_only=True)):
        if (row[0]):   
            partname            = str(row[0])
            instancename        = partname +"-{:02d}".format(idx+1)
            x, y, z = int(row[2]), int(row[3]), int(row[4])

            orientation = row[1]
            if orientation == 'zy':
                normal=(1,0,0)
            elif orientation == 'xy':
                normal=(0,0,1)
            elif orientation== 'xz':
                normal=(0,1,0)

            opening_bib[instancename]=Opening(
                model               = current_model,
                part                = None,
                partname            = partname,       
                instance            = None,
                instancename        = instancename,
                origin              = (x,y,z),
                normal              = normal,
                opening_dimension   = opening_type_bib[partname].opening_dimension
                )
    workbook.close()
    return opening_bib
    
def import_floor_bib(sheet_name, wb_name, start_row, current_model, building_layout):
    workbook = openpyxl.load_workbook(wb_name, data_only=True) 
    sheet = workbook[sheet_name]
    floor_bib={}
    for row in sheet.iter_rows(min_row = start_row, min_col=1, max_col=len(EXCEL_MAP["Floors"].keys()), values_only=True):
        if (row[0]):
            floor_bib=UTIL_modelbuild.FloorTools.instantiate_floor_array(
                current_model       = current_model,
                building_layout     = building_layout,
                floor_bib           = floor_bib,
                floor_data          = format_excel_row(row,EXCEL_MAP["Floors"]),
                )
    workbook.close()
    return floor_bib

def import_layout_bib(sheet_name, wb_name):
    workbook = openpyxl.load_workbook(wb_name, data_only=True)
    sheet = workbook[sheet_name]
    building_layout={}

    for row in list(sheet.rows)[6:9]:
        axes=[] 
        for data in row[1:-1]:
            text=str(data.value)
            subaxes=text.split("|")
            subaxes_to_keep=[]
            for axis in subaxes:
                if axis!='None':
                    subaxes_to_keep.append(float(axis))
            if len(subaxes_to_keep)>0:
                axes.append(subaxes_to_keep)
        building_layout[str(row[0].value)]=axes
    workbook.close()
    return building_layout


def import_conenctor_density(sheet_name,wb_name,start_row):
    workbook = openpyxl.load_workbook(wb_name, data_only=True)
    sheet = workbook[sheet_name]
    densities={}
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=23, values_only=True):
        if row[0]:
            row_data=format_excel_row(row,EXCEL_MAP["Connector density"])
            densities[row_data["Axis"]]=row_data["Axis density"]
    workbook.close()
    return densities


def import_shell_sections(sheet_name, wb_name, start_row, current_model):
    workbook = openpyxl.load_workbook(wb_name, data_only=True)
    sheet = workbook[sheet_name]
    shell_section_bib={}
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=len(EXCEL_MAP["Shell sections"].keys()), values_only=True):
        if row[0]:
            shell_section_bib[str(row[0])]=define_section(
                current_model   = current_model,
                section_data    = format_excel_row(row,EXCEL_MAP["Shell sections"]),
                )
    workbook.close()
    return shell_section_bib
def import_wire_sections(sheet_name, wb_name, start_row, current_model):
    workbook = openpyxl.load_workbook(wb_name, data_only=True)
    sheet = workbook[sheet_name]
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=23, values_only=True):
        if row[0]:
            define_section(
                current_model   = current_model,
                section_data    = format_excel_row(row,EXCEL_MAP["Wire sections"]),
                )
    workbook.close()
def import_columnbeam_bib(sheet_name, wb_name, start_row, current_model):
    workbook = openpyxl.load_workbook(wb_name, data_only=True) 
    sheet = workbook[sheet_name]
    cb_bib = {}
    for row in sheet.iter_rows(min_row = start_row, min_col=1, max_col=20, values_only=True):
        if (row[0]):
            cb_bib=UTIL_modelbuild.ColumnBeamTools.instantiate_columnbeam_object(
                current_model       = current_model,
                cb_bib              = cb_bib,
                wire_data           = format_excel_row(row,EXCEL_MAP["ColumnsBeams"]),
                )
    workbook.close()
    return cb_bib

def import_material_bib(sheet_name, wb_name, start_row, current_model):
    workbook = openpyxl.load_workbook(wb_name, data_only=True, read_only=True)
    sheet = workbook[sheet_name]
    returnbuffer=[]
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=len(EXCEL_MAP["Materials"].keys()), values_only=True):
        if row[0]:
            material_data=format_excel_row(row,EXCEL_MAP["Materials"])
            if material_data["Name"]=="CLT panel (for connector calculation)":
                returnbuffer.append( {
                    "E0,mean"   : material_data["Density"],
                    "G0,mean"   : material_data["E1"],
                })
            else:
                define_material(
                    current_model = current_model,
                    material_data = material_data
                    )
    workbook.close()
    return returnbuffer

def import_connectors(sheet_name, wb_name, start_row, current_model):
    workbook = openpyxl.load_workbook(wb_name, data_only=True)
    sheet = workbook[sheet_name]
    connector_bib={}
    for row in sheet.iter_rows(min_row=start_row, min_col=1, max_col=len(EXCEL_MAP["Connectors"].keys()), values_only=True):
        if row[0]:
            connector_bib=instantiate_connector_section(
                current_model   = current_model,
                connector_bib   = connector_bib,
                connector_data  = format_excel_row(row,EXCEL_MAP["Connectors"]),
                )
    workbook.close()
    return connector_bib

def import_sensor_data_and_mode_shapes(sheet_name, wb_name, start_row):
        workbook = openpyxl.load_workbook(wb_name, data_only=True)
        sheet = workbook[sheet_name]
        sensor_data = {}
        for row in sheet.iter_rows(min_row=start_row, max_row=12, min_col=1, max_col=5,  values_only=True):            
            if row[0]:
                sensor_data[str(row[0])]={
                    "position"  : (float(row[1]),float(row[2]),float(row[3])),
                }
        mode_dict={
            1:[],2:[],3:[],
        }
        for row in sheet.iter_rows(min_row=start_row, max_row=20, min_col=8, max_col=12, values_only=True):
            if row[0]:
                mode_dict[1].append(float(row[0]))
                mode_dict[2].append(float(row[1]))
                mode_dict[3].append(float(row[2]))
        return sensor_data,mode_dict