# -*- coding: mbcs -*-
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
from sys import *
import time
import multiprocessing
def get_database():
    return mdb

def get_model():
    return mdb.models.values()[0]

def rename_model(model_name):
# Rename the last model
    mdb.models.changeKey(
        fromName=mdb.models.keys()[0], 
        toName=model_name
        )

def flush_database():
    myModel = mdb.Model(name='Model')
    for i in range(1,len(mdb.models.keys())):   
        del mdb.models[mdb.models.keys()[i]]
    try:
        for key in mdb.models.sections.keys():
            try:
                del mdb.models[key]
            except:
                pass
    except: 
        pass
def alphabetic_axis_to_index(axis):
    return ord(axis.upper())-ord('A')+1

def get_axis_index(axis_string):
    return axis_string.split("|")
def get_axis_value(layout,dir,axis):
    if axis[0][0] in '01234567890':
        main_index = int(axis[0])-1
        sub_index  = int(axis[1])
    else:
        main_index = int(alphabetic_axis_to_index(axis[0]))-1
        sub_index  = int(axis[1])
    return layout[dir][main_index][sub_index]
def section_header(heading, start_time):
        print(str(round(time.time()-start_time))+"\t"+heading)
def script_header(heading):
        print(":"*30+heading+":"*30)
    
def log_progress(path,progress_text):
    import datetime
    f = open(path+"\\log\\worklog.txt",'a')
    time=datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    f.write(time+"\t"+progress_text+'\n')
    f.close()

def create_steps(
    current_model=[],
    steps=[]
):  
    for step_name in current_model.steps.keys():
        try:
            del current_model.steps[step_name]
        except:
            pass
    current_model.StaticStep(
        initialInc=1.0, 
        name='Static', 
        previous='Initial'
        )
    current_model.FrequencyStep(   
        numEigen=5,   
        maxEigen=10.0, 
        name='Frequency', 
        previous='Static'
        )
    current_model.ComplexFrequencyStep(
        numEigen=5,   
        maxEigen=10.0, 
        name='Complex frequency', 
        previous='Frequency'
        )
    for stepname, run_step in steps.items():
        if not run_step:
            current_model.steps[stepname].suppress()


def create_loads(current_model=[]):
    current_model.Gravity(
        comp2=-9.81*1000, 
        createStepName='Static', 
        distributionType=UNIFORM, 
        field='', 
        name='Gravity'
        )
def create_job(
    name_of_job   = [],
    name_of_model = []
):
    import os
    if os.path.exists(name_of_job+".lck"):
        os.remove(name_of_job+".lck")
    mdb.Job(
        atTime=None, 
        contactPrint=OFF, 
        description='', 
        echoPrint=OFF, 
        explicitPrecision=SINGLE, 
        getMemoryFromAnalysis=True, 
        historyPrint=OFF, 
        memory=90, 
        memoryUnits=PERCENTAGE, 
        model=name_of_model, 
        modelPrint=OFF, 
        multiprocessingMode=MPI, 
        name=name_of_job, 
        nodalOutputPrecision=SINGLE, 
        numCpus=multiprocessing.cpu_count(), 
        numDomains=multiprocessing.cpu_count(), 
        numGPUs=1, 
        queue=None, 
        resultsFormat=ODB, 
        scratch='', 
        type=ANALYSIS, 
        userSubroutine='', 
        waitHours=0, 
        waitMinutes=0)

def submit_job(
    name_of_job='Job'): 
    mdb.jobs[name_of_job].submit(consistencyChecking=OFF)
    mdb.jobs[name_of_job].waitForCompletion()