# -*- coding: mbcs -*-
import math
from CONF_config                import *
from UTIL_model                 import *
from CLASS_classes              import *
from UTIL_part                  import *
from UTIL_assembly              import *
from UTIL_tools                 import *


class WallTools:
    @staticmethod
    def instantiate_wall_array(
        current_model       = [],
        building_layout     = [],
        wall_bib            = [],
        wall_data           = [],
    ):  
        y_start = get_axis_index(wall_data["Storey_base"])
        y_stop  = get_axis_index(wall_data["Storey_top"])
        follow  = get_axis_index(wall_data["Axis_follow"])
        start   = get_axis_index(wall_data["Axis_start"])
        stop    = get_axis_index(wall_data["Axis_stop"])
        wall_height = get_axis_value(building_layout,'Y Direction',y_stop)-get_axis_value(building_layout,'Y Direction',y_start)
        if wall_data["Axis_follow"][0][0] in '0123456789':
            dir_X           = 0
            dir_Y           = 0
            dir_Z           = 1
            ori_X           = get_axis_value(building_layout,'X Direction',follow)
            ori_Y           = get_axis_value(building_layout,'Y Direction',y_start)
            ori_Z           = get_axis_value(building_layout,'Z Direction',start) 
            length          = get_axis_value(building_layout,'Z Direction',stop)-get_axis_value(building_layout,'Z Direction',start)
        else: 
            dir_X           = 1
            dir_Y           = 0
            dir_Z           = 0
            ori_X           = get_axis_value(building_layout,'X Direction',start)
            ori_Y           = get_axis_value(building_layout,'Y Direction',y_start)
            ori_Z           = get_axis_value(building_layout,'Z Direction',follow)
            length          = get_axis_value(building_layout,'X Direction',stop)-get_axis_value(building_layout,'X Direction',start)
        if wall_data["Element width"]:
            num_elements    = int(math.ceil(length/wall_data["Element width"]))
        else:
            num_elements=1
        element_width       = length/num_elements

        for i in range(num_elements):
            counter=i+1
            name=wall_data["Name"]+"-{:02d}".format(counter)
            X               = ori_X + (counter-1)*dir_X*element_width
            Y               = ori_Y + (counter-1)*dir_Y*element_width
            Z               = ori_Z + (counter-1)*dir_Z*element_width

            wall_bib[name]=WallInstance(
                    model           = current_model,
                    part            = [],
                    partname        = name,                   
                    instance        = [],
                    instancename    = name,
                    origin          = (X,Y,Z),
                    direction       = (dir_X,dir_Y,dir_Z),
                    width           = element_width,
                    height          = wall_height,
                    section         = wall_data["Section"],
                    foundation      = wall_data["isFoundation?"],
                    distributed_mass= wall_data["Distributed mass"],
                    mesh_controls   = {
                        "Global mesh size avg"      : wall_data["Global mesh size avg"],
                        "Global mesh size min"      : wall_data["Global mesh size min"],
                        "Bounding edge mesh size"   : wall_data["Bounding edge mesh size"],
                        "Internal edge mesh size"   : wall_data["Internal edge mesh size"],
                        "Element type"              : wall_data["Element type"],
                        "Face structure"            : wall_data["Face structure"],
                    },
                )
        return wall_bib
    @staticmethod
    def generate_walls(wall_bib):
        for wall_object in wall_bib.values():
            WallTools.generate_wall_element(wall_object)
    @staticmethod        
    def generate_wall_element(
            wall_object,
        ):
        #Only orthogonal angles currently, change the code logic if needed
        rotation_angle = 0 if wall_object.length_X else -90

        part=define_shell_part(     
            current_model   = wall_object.model,
            part_name       = wall_object.partname, 
            width           = (wall_object.length_X**2+wall_object.length_Z**2)**0.5,
            height          = wall_object.length_Y
        )
        create_instance(
            current_model   = wall_object.model, 
            instancename    = wall_object.instancename,
            part            = part,
        )
        move_instance(
            current_model   = wall_object.model,
            instancename    = wall_object.instancename,
            translation     = (wall_object.X,wall_object.Y,wall_object.Z),
            rotation_angle  = rotation_angle,
            rotation_axis   = (0,1,0)
        )


class FloorTools:
    @staticmethod
    def instantiate_floor_array(
        current_model           = [], 
        building_layout         = [], 
        floor_bib               = [],
        floor_data              = []
    ):   
        storey      = get_axis_index(floor_data["Storey_base"]) 
        X_start     = get_axis_index(floor_data["X_start"])
        X_stop      = get_axis_index(floor_data["X_stop"])
        Z_start     = get_axis_index(floor_data["Z_start"])
        Z_stop      = get_axis_index(floor_data["Z_stop"])
        
        width_x         = (get_axis_value(building_layout,'X Direction',X_stop)-get_axis_value(building_layout,'X Direction',X_start))/floor_data["Num_X"]
        width_z         = (get_axis_value(building_layout,'Z Direction',Z_stop)-get_axis_value(building_layout,'Z Direction',Z_start))/floor_data["Num_Z"]
        storey_height   = get_axis_value(building_layout,'Y Direction',storey)
        for row in range(floor_data["Num_Z"]):
            for col in range(floor_data["Num_X"]):
                X               = 0 + (col)*width_x
                Y               = storey_height
                Z               = 0 + (row)*width_z
                name=floor_data["Name"] + "-{:02d}".format(row+1)+ "|{:02d}".format(col+1)

                floor_bib[name]=FloorInstance(
                        model           = current_model,
                        part            = [],
                        partname        = name,                               
                        instance        = [],
                        instancename    = name,
                        origin          = (X,Y,Z),
                        width_x         = width_x,
                        width_z         = width_z,
                        section         = floor_data["Section"],
                        foundation      = False,
                        distributed_mass= floor_data["Distributed mass"],
                        mesh_controls   = {
                            "Global mesh size avg"      : floor_data["Global mesh size avg"],
                            "Global mesh size min"      : floor_data["Global mesh size min"],
                            "Bounding edge mesh size"   : floor_data["Bounding edge mesh size"],
                            "Internal edge mesh size"   : floor_data["Internal edge mesh size"],
                            "Element type"              : floor_data["Element type"],
                            "Face structure"            : floor_data["Face structure"],
                        },
                    )
        return floor_bib
    @staticmethod
    def generate_floors(floor_bib):
        for floor_object in floor_bib.values():
            FloorTools.generate_floor_element(floor_object)
    @staticmethod
    def generate_floor_element(floor_object):
        #Create origin part
        part=define_shell_part(
            current_model   = floor_object.model,
            part_name       = floor_object.partname, 
            width           = floor_object.length_X,
            height          = floor_object.length_Z,
            )
        create_instance(
            current_model   = floor_object.model,
            instancename    = floor_object.instancename,
            part            = part,
            )
        #Move it into position
        move_instance(
            current_model   = floor_object.model,
            instancename    = floor_object.instancename,
            translation     = (0, floor_object.Y, 0),
            rotation_angle  = 90,
            rotation_axis   = (1,0,0)
            )
    @staticmethod
    def create_floor_panel_array(
        current_model   = [],
        origin_part     = [],
        direction_1     = (1.0, 0.0, 0.0),
        direction_2     = (0.0, 0.0, 1.0),
        num_dir1        = [],
        num_dir2        = [],
        spacing_1       = [],
        spacing_2       = [],
    ):
        current_model.rootAssembly.LinearInstancePattern(
            direction1=direction_1, 
            direction2=direction_2, 
            instanceList=(origin_part, ), 
            number1=num_dir1, 
            number2=num_dir2, 
            spacing1=spacing_1, 
            spacing2=spacing_2
            )


class ColumnBeamTools:
    @staticmethod
    def instantiate_columnbeam_object(
    current_model       = [],
    cb_bib              = {},
    wire_data           = {},
    ):

        cb_bib[wire_data["Name"]]=WireInstance(
                model           = current_model,
                part            = None,
                partname        = wire_data["Name"],           
                instance        = None,
                instancename    = wire_data["Name"],
                origin          = (wire_data["X_coord"],wire_data["Y_coord"],wire_data["Z_coord"]),
                lengths         = (wire_data["X_length"],wire_data["Y_length"],wire_data["Z_length"]),
                section         = wire_data["Section"],
                foundation      = wire_data["isFoundation?"],
                distributed_mass= wire_data["Distributed mass"],
                mesh_controls   = {
                    "Global mesh size avg"      : wire_data["Global mesh size avg"],
                    "Global mesh size min"      : wire_data["Global mesh size min"],
                    "Bounding edge mesh size"   : wire_data["Bounding edge mesh size"],
                    "Internal edge mesh size"   : wire_data["Internal edge mesh size"],
                    "Element type"              : wire_data["Element type"],
                    "Face structure"            : wire_data["Face structure"],
                },
            )
        return cb_bib

    @staticmethod
    def generate_columnbeam_element(columnbeam_object):
        length = (columnbeam_object.length_X**2+columnbeam_object.length_Y**2+columnbeam_object.length_Z**2)**0.5
        part=define_wire_part(
            current_model       = columnbeam_object.model,
            name                = columnbeam_object.partname,   
            length              = length  
            )

        create_instance(
            current_model       = columnbeam_object.model,
            instancename        = columnbeam_object.instancename,
            part                = part
            )

        theta_x = math.atan(columnbeam_object.length_X/length)
        theta_z = math.atan(columnbeam_object.length_Z/length)   
        move_instance(
            current_model   = columnbeam_object.model,
            instancename    = columnbeam_object.instancename,
            translation     = (0,0,0),
            rotation_angle  = (theta_x),
            rotation_axis   = (0,0,1),
            )
        move_instance(
            current_model   = columnbeam_object.model,
            instancename    = columnbeam_object.instancename,
            translation     = (0,0,0),
            rotation_angle  = (theta_z),
            rotation_axis   = (-1,0,0),
            )
        move_instance(
            current_model   = columnbeam_object.model,
            instancename    = columnbeam_object.instancename,
            translation     = (columnbeam_object.X,columnbeam_object.Y,columnbeam_object.Z),
            )
    @staticmethod
    def generate_columns_and_beams(cb_bib):
        for cb_object in cb_bib.values():
            ColumnBeamTools.generate_columnbeam_element(cb_object)


class Openingtools:
    @staticmethod
    def get_cut_dictionary(opening_bib,cut_from_bib):
        cut_dictionary = {}
        for openingname,opening in opening_bib.items():
            instances_to_be_cut=[]
            for coord in opening.get_bounding_box_coords():
                affected_instances=[(instance_object_key,instance_object) for instance_object_key,instance_object in cut_from_bib.items() if instance_object.detect_overlap(coord)>=2]
                [instances_to_be_cut.append((instance_object_key,instance_object)) for instance_object_key,instance_object in affected_instances if not (instance_object_key,instance_object) in instances_to_be_cut]
            cut_dictionary[opening]=instances_to_be_cut
        return cut_dictionary
    @staticmethod
    def generate_opening_element(opening_object):
        rotation_angle=0
        if opening_object.normal == (1,0,0):
            rotation_angle  = -90
            rotation_axis   = (0,1,0)
        elif opening_object.normal == (0,0,1):
            rotation_angle  = 0
            rotation_axis   = (0,1,0)
        elif opening_object.normal == (0,1,0):
            rotation_angle  = 90
            rotation_axis   = (1,0,0)

        create_instance(
            current_model   = opening_object.model,
            instancename    = opening_object.instancename,
            part            = opening_object.model.parts[opening_object.partname],
            )  
        move_instance(
            current_model   = opening_object.model,
            instancename    = opening_object.instancename,
            translation     = (opening_object.X,opening_object.Y,opening_object.Z),
            rotation_angle  = rotation_angle, 
            rotation_axis   = rotation_axis
            )
    @staticmethod
    def generate_opening_parts(opening_type_bib):
        for opening_type in opening_type_bib.values():
            define_shell_part(
                current_model   = opening_type.model,
                part_name       = opening_type.partname,
                width           = opening_type.opening_dimension[0],
                height          = opening_type.opening_dimension[1]
                )
    @staticmethod
    def generate_openings(opening_bib):
        for opening_object in opening_bib.values():
            Openingtools.generate_opening_element(opening_object)
    @staticmethod
    def cut_opening(
        current_model       = [],
        cut_from_object     = [],
        opening             = [],
        modified_suffix     = "-mod"
    ):

        current_model.rootAssembly.InstanceFromBooleanCut(
            cuttingInstances=(opening.instance, ), 
            instanceToBeCut=cut_from_object.instance, 
            name=cut_from_object.partname+modified_suffix, 
            originalInstances=SUPPRESS)
        del current_model.rootAssembly.instances[cut_from_object.instancename]
    
        current_model.rootAssembly.instances.changeKey(
            fromName=cut_from_object.partname+modified_suffix+"-1", 
            toName=cut_from_object.instancename
        )

        cut_from_object.instance=current_model.rootAssembly.instances[cut_from_object.instancename]
        cut_from_object.part=current_model.parts[cut_from_object.partname+modified_suffix]

        return cut_from_object
    @staticmethod
    def cut_openings(current_model,all_bib,cut_dictionary,modified_suffix):
        #Suppressing the model speeds things up
        current_model.rootAssembly.suppressFeatures(tuple(current_model.rootAssembly.features.keys()))
        #Here we loop through all the openings
        for idx,(opening,cut_list) in enumerate(cut_dictionary.items()):
            print("\t\t\tCutting opening "+str(idx+1)+"/"+str(len(cut_dictionary.keys())))
            #Loop through all the walls in the cut_list
            for instance_object_key,instance_object in cut_list:
                try:
                    current_model.rootAssembly.resumeFeatures(tuple([instance_object.instancename, opening.instancename]))
                    all_bib[instance_object_key]=Openingtools.cut_opening(current_model,instance_object,opening,modified_suffix)
                except:
                    print("Could not cut" + opening.instancename + "from" + instance_object.instancename)
            #Delete the opening instance once we are done with it
            del current_model.rootAssembly.instances[opening.instancename]
        current_model.rootAssembly.resumeFeatures(tuple(current_model.rootAssembly.features.keys()))
        return all_bib