from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
import math

def create_datumplane_array(current_model, normal, step, length):
    count=int(math.ceil(length/step))
    step=length/count

    if normal ==(1,0,0):
        principal_plane=YZPLANE
    elif normal ==(0,1,0):
        principal_plane=XZPLANE  
    elif normal ==(0,0,1):
        principal_plane=XYPLANE

    for i in range(1,count):
        current_model.rootAssembly.DatumPlaneByPrincipalPlane(
            offset=(0.0+step*i), 
            principalPlane=principal_plane
            )
def partition_edges_by_plane_array(
        current_model=[],
        plane_array=[],
        instance_bib=[],
        ):
    normal=plane_array.values()[0].normal
    for coord,datumplane in plane_array.items():
        coordinate=(normal[0]*coord,normal[1]*coord,normal[2]*coord)
        edges_to_partition=[]
        for instance in instance_bib.values():
            if instance.is_within_projection(coordinate):
                for edge in instance.get_instance_edges():
                    edges_to_partition.append(edge.edge_object)
        if len(edges_to_partition)>0:
            try:
                current_model.rootAssembly.PartitionEdgeByDatumPlane(
                    datumPlane=datumplane, 
                    edges= edges_to_partition
                    )
            except:
                print("Couldnt partition by datumplane at " + str(coord) + " along array")
                pass
    
def partition_faces_by_plane_array(
        current_model=[],
        plane_array=[],
        instance_bib=[],
        ):
    normal=plane_array.values()[0].normal
    for coord,datumplane in plane_array.items():
        coordinate=(normal[0]*coord,normal[1]*coord,normal[2]*coord)
        faces_to_partition=[]
        for instancename,instance in instance_bib.items():
            if instance.is_within_projection(coordinate):
                for face in current_model.rootAssembly.instances[instancename].faces:
                    faces_to_partition.append(face)
        if len(faces_to_partition)>0:
            try:
                partition=mdb.models['SML3'].rootAssembly.PartitionFaceByDatumPlane(
                    datumPlane=datumplane, 
                    faces=faces_to_partition
                )
            except:
                print("Couldnt partition by datumplane at " + str(coord) + " along array")
                pass
def partition_t_intersects(
    current_model,
    t_intersect_bib
    ):
    for wall_to_be_intersected, intersecting_walls in t_intersect_bib.items():
        for intersecting_wall in intersecting_walls:
            try:
                current_model.rootAssembly.PartitionFaceByIntersectFace(
                    cuttingFaces=current_model.rootAssembly.instances[intersecting_wall].faces,
                    faces=current_model.rootAssembly.instances[wall_to_be_intersected].faces
                    )
            except:
                print("Couldnt partition t-insterected: ",wall_to_be_intersected," by intersecting ",intersecting_wall)
                pass

def partition_point_intersects(
    current_model=[],
    all_bib={},
    cb_bib={},
    all_point_intersects={}
):
    for intersected_shell, intersecting_wires in all_point_intersects.items():
        for intersecting_wire in intersecting_wires:
            intersection_bounding_box=cb_bib[intersecting_wire].get_bounding_box_span()
            for n in range(2):
                x_coord = intersection_bounding_box[0][n]
                y_coord = intersection_bounding_box[1][n]
                z_coord = intersection_bounding_box[2][n]
                if all_bib[intersected_shell].detect_overlap((x_coord,y_coord,z_coord)):
                    step=300
                    nodes=[]
                    for u1 in range(-1,2,2):
                        for u2 in range(-1,2,2):
                                    nodes.append((x_coord+u1*step, y_coord+u2*step, z_coord+00*step))  
                                    nodes.append((x_coord+u1*step, y_coord+00*step, z_coord+u2*step))
                                    nodes.append((x_coord+00*step, y_coord+u1*step, z_coord+u2*step))
                    partition_lines=[(nodes[i],nodes[j]) for i in range(len(nodes)) for j in range(i, len(nodes))]   
                                            
                    partition_point_pair_cloud(current_model,current_model.rootAssembly.instances[intersected_shell].faces,partition_lines)

                    

def partition_point_pair_cloud(
    current_model,
    faces,
    partition_lines
):
    for point_a,point_b in partition_lines:
        try:
            current_model.rootAssembly.PartitionFaceByShortestPath(
                faces=faces,
                point1=point_a, 
                point2=point_b
                )
        except:
            pass   



